package me.totalfreedom.totalfreedommod.command;

import me.totalfreedom.totalfreedommod.player.FPlayer;
import me.totalfreedom.totalfreedommod.punishments.Punishment;
import me.totalfreedom.totalfreedommod.punishments.PunishmentType;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.HumanEntity;
import org.bukkit.entity.Player;

import java.util.List;
import java.util.Optional;

@CommandPermissions(permission = "blockedit", source = SourceType.BOTH)
@CommandParameters(description = "Restricts/restores block modification abilities for everyone on the server or a certain player.", usage = "/<command> [<player> [reason] | list | purge | all]")
public class Command_blockedit extends FreedomCommand
{
    @Override
    public boolean run(final CommandSender sender, final Player playerSender, final Command cmd, final String commandLabel, String[] args, final boolean senderIsConsole)
    {
        if (args.length > 0)
        {
            switch (args[0].toLowerCase())
            {
                case "list" ->
                {
                    List<? extends Player> list = server.getOnlinePlayers().stream().filter(player ->
                            plugin.pl.getPlayer(player).isEditBlocked()).sorted().toList();

                    boolean plural = list.size() != 1;
                    msgNew("There <grammar> <count> player<plural> online with restricted block modification abilities: <players>",
                            Placeholder.unparsed("grammar", plural ? "are" : "is"),
                            Placeholder.unparsed("count", String.valueOf(list.size())),
                            Placeholder.unparsed("plural", plural ? "s" : ""),
                            Placeholder.unparsed("colon", list.size() > 0 ? ":" : "."),
                            Placeholder.unparsed("players", FUtil.listToString(list.stream().map(HumanEntity::getName).toList())));
                }
                case "purge" ->
                {
                    FUtil.adminAction(sender.getName(), "Restoring block modification abilities for all players", true);

                    List<? extends Player> list = server.getOnlinePlayers().stream().filter(player ->
                            plugin.pl.getPlayer(player).isEditBlocked()).toList();

                    list.forEach(player ->
                    {
                        plugin.pl.getPlayer(player).setEditBlocked(false);
                        msgNew(player, "<green>Your block modification abilities have been restored.");
                    });

                    msgNew("Restored block modification abilities for <count> player<plural>.",
                            Placeholder.unparsed("count", String.valueOf(list.size())),
                            Placeholder.unparsed("plural", list.size() != 1 ? "s" : ""));
                }
                case "all", "-a" ->
                {
                    FUtil.adminAction(sender.getName(), "Restricting block modification abilities for all non-admins", true);

                    List<? extends Player> list = server.getOnlinePlayers().stream().filter(player ->
                            !plugin.al.isAdmin(player)).toList();

                    list.forEach(player ->
                    {
                        plugin.pl.getPlayer(player).setEditBlocked(true);
                        msgNew(player, "<red>Your block modification abilities have been restricted.");
                    });

                    msgNew("Restricted block modification abilities for <count> player<plural>.",
                            Placeholder.unparsed("count", String.valueOf(list.size())),
                            Placeholder.unparsed("plural", list.size() != 1 ? "s" : ""));
                }
                default -> Optional.ofNullable(getPlayer(args[0])).ifPresentOrElse(player ->
                {
                    FPlayer fPlayer = plugin.pl.getPlayer(player);

                    if (fPlayer.isEditBlocked())
                    {
                        FUtil.adminAction(sender.getName(), "Restoring block modification abilities for " + player.getName(), true);
                        fPlayer.setEditBlocked(false);
                        msgNew("Restored block modification abilities for <player>.", Placeholder.unparsed("player", player.getName()));
                        msgNew(player, "<green>Your block modification abilities have been restored.");
                    }
                    else
                    {
                        if (plugin.al.isAdmin(player))
                        {
                            msgNew("<red><player> is an admin, and as such cannot have their block modification abilities restricted.", Placeholder.unparsed("player", player.getName()));
                        }
                        else
                        {
                            FUtil.adminAction(sender.getName(), "Restricting block modification abilities for " + player.getName(), true);
                            fPlayer.setEditBlocked(true);
                            msgNew("Restricted block modification abilities for <player>.", Placeholder.unparsed("player", player.getName()));
                            msgNew(player, "<red>Your block modification abilities have been restricted.");

                            plugin.pul.logPunishment(new Punishment(player.getName(), FUtil.getIp(player),
                                    sender.getName(), PunishmentType.BLOCKEDIT, null));
                        }
                    }

                }, () -> msg(PLAYER_NOT_FOUND));
            }

            return true;
        }
        else
        {
            return false;
        }
    }
}