package me.totalfreedom.totalfreedommod.command;

import me.totalfreedom.totalfreedommod.punishments.Punishment;
import me.totalfreedom.totalfreedommod.punishments.PunishmentType;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.*;

@CommandPermissions(permission = "mute", source = SourceType.BOTH)
@CommandParameters(description = "Mutes a player with brute force.", usage = "/<command> <[-q] <player> [reason] | list | purge | all>", aliases = "stfu")
public class Command_mute extends FreedomCommand
{
    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (args.length == 0)
        {
            return false;
        }

        switch (args[0].toLowerCase())
        {
            case "list" ->
            {
                List<String> muted = server.getOnlinePlayers().stream().filter(player -> plugin.pl.getPlayer(player).isMuted()).map(player -> player.getName()).toList();

                if (muted.isEmpty())
                {
                    msgNew("Nobody is currently muted.");
                } else
                {
                    msgNew("Muted players: <players>", Placeholder.unparsed("players", FUtil.listToString(muted)));
                }
            }
            case "purge" ->
            {
                FUtil.adminAction(sender.getName(), "Unmuting all players", true);
                List<? extends Player> list = server.getOnlinePlayers().stream().filter(player ->
                        plugin.pl.getPlayer(player).isMuted()).toList();

                list.forEach(player ->
                {
                    plugin.pl.getPlayer(player).setMuted(false);
                    FUtil.playerTitle(player, "<red>You have been unmuted.", "<yellow>Be sure to follow the rules!");
                });

                msgNew("Unmuted <count> player<plural>.",
                        Placeholder.unparsed("count", String.valueOf(list.size())),
                        Placeholder.unparsed("plural", list.size() != 1 ? "s" : ""));
            }
            case "all" ->
            {
                FUtil.adminAction(sender.getName(), "Muting all non-admins", true);
                List<? extends Player> list = server.getOnlinePlayers().stream().filter(player ->
                        !plugin.al.isAdmin(player)).toList();

                list.forEach(player ->
                {
                    plugin.pl.getPlayer(player).setMuted(true);
                    FUtil.playerTitle(player, "<red>You have been globally muted.", "<yellow>Please be patient and you will be unmuted shortly.");
                });

                msgNew("Muted <count> player<plural>.",
                        Placeholder.unparsed("count", String.valueOf(list.size())),
                        Placeholder.unparsed("plural", list.size() != 1 ? "s" : ""));
            }
            default ->
            {
                boolean quiet = args[0].equalsIgnoreCase("-q");

                // Handling the -q parameter
                if (quiet)
                {
                    if (args.length == 1) return false;
                    args = ArrayUtils.subarray(args, 1, args.length);
                }

                // Handling the (optional) reason
                String reason = args.length > 1 ? StringUtils.join(args, " ", 1, args.length) : null;

                // Showtime
                Optional.ofNullable(getPlayer(args[0])).ifPresentOrElse(player ->
                {
                    if (plugin.al.isAdmin(player))
                    {
                        msgNew("<red><player> is an admin, and as such can't be muted.", Placeholder.unparsed("player", player.getName()));
                        return;
                    } else if (plugin.pl.getPlayer(player).isMuted())
                    {
                        msgNew("<red><player> is already muted.", Placeholder.unparsed("player", player.getName()));
                        return;
                    }

                    // Don't broadcast the mute if it was quiet
                    if (!quiet)
                    {
                        FUtil.adminAction(sender.getName(), "Muting " + player.getName(), true);
                    }

                    // Mutes the player
                    plugin.pl.getPlayer(player).setMuted(true);

                    // Notify the player that they have been muted
                    FUtil.playerTitle(player, "<red>You have been muted.", "<yellow>Be sure to follow the rules!");
                    msgNew(player, "<red>You have been muted by <yellow><sender>", Placeholder.unparsed("sender", sender.getName()));

                    // Give them the reason if one is present.
                    if (reason != null)
                    {
                        msgNew(player, "<red>Reason: <yellow><reason>", Placeholder.unparsed("reason", reason));
                    }

                    if (quiet)
                    {
                        msgNew("Muted <player>.", Placeholder.unparsed("player", player.getName()));
                    } else
                    {
                        msgNew("Quietly muted <player>.", Placeholder.unparsed("player", player.getName()));
                    }

                    plugin.pul.logPunishment(new Punishment(player.getName(), FUtil.getIp(player), sender.getName(),
                            PunishmentType.MUTE, reason));
                }, () -> msg(PLAYER_NOT_FOUND));
            }
        }

        return true;
    }

    @Override
    public List<String> getTabCompleteOptions(CommandSender sender, Command command, String alias, String[] args)
    {
        if (!plugin.al.isAdmin(sender))
        {
            return null;
        }

        if (args.length == 1)
        {
            List<String> arguments = new ArrayList<>();
            arguments.addAll(FUtil.getPlayerList());
            arguments.addAll(Arrays.asList("list", "purge", "all"));
            return arguments;
        }

        return Collections.emptyList();
    }
}
