package me.totalfreedom.totalfreedommod.command;

import io.papermc.lib.PaperLib;
import me.totalfreedom.totalfreedommod.player.FPlayer;
import me.totalfreedom.totalfreedommod.player.PlayerData;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;

import java.util.HashMap;
import java.util.Map;

@CommandPermissions(permission = "ride", source = SourceType.ONLY_IN_GAME)
@CommandParameters(description = "Ride on the top of the specified player.", usage = "/<command> <playername | mode <normal | off | ask>>")
public class Command_ride extends FreedomCommand
{

    private final Map<Player, Player> RIDE_REQUESTS = new HashMap<>(); // requested, requester

    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        final FPlayer fPlayer = plugin.pl.getPlayer(playerSender);
        if (fPlayer.getCageData().isCaged())
        {
            msgNew("<red>You cannot use this command while caged.");
            return true;
        }

        if (args.length < 1)
        {
            return false;
        }

        if (args[0].equalsIgnoreCase("accept") || args[0].equalsIgnoreCase("yes"))
        {
            if (!RIDE_REQUESTS.containsKey(playerSender))
            {
                msgNew("<red>You don't have any pending requests.");
                return true;
            }

            Player requester = RIDE_REQUESTS.get(playerSender);
            if (requester == null)
            {
                msgNew("<red>The player who sent the request is no longer online.");
                RIDE_REQUESTS.remove(playerSender);
                return true;
            }

            msgNew("Request accepted.");
            msgNew(requester, "<green>Your request has been accepted.");

            if (requester.getWorld() != playerSender.getWorld())
            {
                PaperLib.teleportAsync(requester, playerSender.getLocation());
            }

            RIDE_REQUESTS.remove(playerSender);
            playerSender.addPassenger(requester);
            return true;
        }

        if (args[0].equalsIgnoreCase("deny") || args[0].equalsIgnoreCase("no"))
        {
            if (!RIDE_REQUESTS.containsKey(playerSender))
            {
                msgNew("<red>You don't have any pending requests.");
                return true;
            }
            Player requester = RIDE_REQUESTS.get(playerSender);
            if (requester == null)
            {
                msgNew("<red>The player who sent the request is no longer online.");
                RIDE_REQUESTS.remove(playerSender);
                return true;
            }
            msgNew("Request denied.");
            RIDE_REQUESTS.remove(playerSender);
            msgNew(requester, "<red>Your request has been denied.");
            return true;
        }

        if (args.length >= 2 && args[0].equalsIgnoreCase("mode"))
        {
            try
            {
                PlayerData.RideMode mode = PlayerData.RideMode.valueOf(args[1].toUpperCase());
                PlayerData playerDataSender = plugin.pl.getData(playerSender);
                playerDataSender.setRideMode(mode);
                plugin.pl.save(playerDataSender);
                msgNew("Ride mode is now set to <mode>.", Placeholder.unparsed("mode", mode.name().toLowerCase()));
            } catch (IllegalArgumentException ex)
            {
                msgNew("<red>Invalid mode.");
            }

            return true;
        }

        final Player player = getPlayer(args[0], true);
        if (player == null)
        {
            msg(PLAYER_NOT_FOUND);
            return true;
        }

        final PlayerData playerData = plugin.pl.getData(player);

        if (player == playerSender)
        {
            msgNew("<red>You can't ride yourself.");
            return true;
        }

        if (playerData.getRideMode() == PlayerData.RideMode.OFF && !isAdmin(sender))
        {
            msgNew("<red>That player cannot be ridden.");
            return true;
        }

        if (playerData.getRideMode() == PlayerData.RideMode.ASK && !FUtil.isExecutive(playerSender.getName()))
        {
            msgNew("<green>Sent a request to the player.");
            msgNew(player, "<aqua><player> has requested to ride you.", Placeholder.unparsed("player", sender.getName()));
            msgNew(player, "<aqua>Type <green>/ride accept</green> to allow the player to ride you.");
            msgNew(player, "<aqua>Type <red>/ride deny</red> to deny their request.");
            msgNew(player, "<aqua>This request will expire in 30 seconds.");
            RIDE_REQUESTS.put(player, playerSender);

            new BukkitRunnable()
            {
                public void run()
                {
                    if (!RIDE_REQUESTS.containsKey(player))
                    {
                        return;
                    }

                    RIDE_REQUESTS.remove(player);
                    msgNew(playerSender, "<red>Your request to ride <player> has automatically expired.", Placeholder.unparsed("player", player.getName()));
                    msgNew(player, "<red>Request expired.");
                }
            }.runTaskLater(plugin, 20 * 30);
            return true;
        }

        if (player.getWorld() != playerSender.getWorld())
        {
            PaperLib.teleportAsync(playerSender, player.getLocation());
        }

        player.addPassenger(playerSender);
        msgNew(player, "<player> is now riding you, run /eject to eject them.", Placeholder.unparsed("player", sender.getName()));
        return true;
    }
}