package me.totalfreedom.totalfreedommod.player;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import me.totalfreedom.totalfreedommod.api.ShopItem;
import me.totalfreedom.totalfreedommod.util.FConverter;
import me.totalfreedom.totalfreedommod.sql.ResultSetProvider;
import me.totalfreedom.totalfreedommod.util.FLog;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import net.kyori.adventure.text.serializer.legacy.LegacyComponentSerializer;
import org.apache.commons.lang.StringUtils;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

public class PlayerData
{
    private final List<String> ips = Lists.newArrayList();
    private final List<String> notes = Lists.newArrayList();
    private UUID uuid;
    private Component tag = Component.empty();
    private String discordID = null;
    private Boolean masterBuilder = false;


    private RideMode rideMode = RideMode.ASK;


    private int coins;
    private List<String> items = Lists.newArrayList();


    private int totalVotes;

    private boolean displayDiscord = true;


    private String loginMessage;

    private Boolean inspect = false;

    public PlayerData(ResultSetProvider resultSet)
    {
        try
        {
            uuid = UUID.fromString(resultSet.getString("uuid"));
            ips.clear();
            ips.addAll(FUtil.stringToList(resultSet.getString("ips")));
            notes.clear();
            notes.addAll(FUtil.stringToList(resultSet.getString("notes")));
            tag = FUtil.miniMessage(resultSet.getString("tag"));
            discordID = resultSet.getString("discord_id");
            masterBuilder = resultSet.getBoolean("master_builder");
            rideMode = RideMode.valueOf(resultSet.getString("ride_mode").toUpperCase());
            coins = resultSet.getInt("coins");
            items.clear();
            items.addAll(FUtil.stringToList(resultSet.getString("items")));
            totalVotes = resultSet.getInt("total_votes");
            displayDiscord = resultSet.getBoolean("display_discord");
            //--
            String tempLoginMessage = resultSet.getString("login_message");
            if (!Strings.isNullOrEmpty(tempLoginMessage) && FConverter.needsConversion(tempLoginMessage))
            {
                tempLoginMessage = FConverter.convertLoginMessage(tempLoginMessage);
            }
            loginMessage = tempLoginMessage;
            //--
            inspect = resultSet.getBoolean("inspect");
        } catch (Throwable e)
        {
            FLog.severe("Failed to load player: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public PlayerData(Player player)
    {
        this.uuid = player.getUniqueId();
    }

    @Override
    public String toString()
    {
        return "Player: " + getName() + "\n" +
                "- IPs: " + StringUtils.join(ips, ", ") + "\n" +
                "- Discord ID: " + discordID + "\n" +
                "- Master Builder: " + masterBuilder + "\n" +
                "- Coins: " + coins + "\n" +
                "- Total Votes: " + totalVotes + "\n" +
                "- Display Discord: " + displayDiscord + "\n" +
                "- Tag: " + tag.toString() + ChatColor.GRAY + "\n" +
                "- Ride Mode: " + rideMode.name() + "\n" +
                "- Login Message: " + loginMessage;
    }

    public List<String> getIps()
    {
        return Collections.unmodifiableList(ips);
    }

    public boolean hasLoginMessage()
    {
        return loginMessage != null && !loginMessage.isEmpty();
    }

    public boolean addIp(String ip)
    {
        if (ips.contains(ip))
        {
            return false;
        }
        ips.add(ip);
        return true;
    }

    public void removeIp(String ip)
    {
        ips.remove(ip);
    }

    public void clearIps()
    {
        ips.clear();
    }

    public void addIps(List<String> ips)
    {
        this.ips.addAll(ips);
    }

    public List<String> getNotes()
    {
        return Collections.unmodifiableList(notes);
    }

    public void clearNotes()
    {
        notes.clear();
    }

    public void addNote(String note)
    {
        notes.add(note);
    }

    public boolean removeNote(int id) throws IndexOutOfBoundsException
    {
        try
        {
            notes.remove(id);
        } catch (IndexOutOfBoundsException e)
        {
            return false;
        }
        return true;
    }

    public void giveItem(ShopItem item)
    {
        items.add(item.getDataName());
    }

    public List<String> getItems()
    {
        return Collections.unmodifiableList(items);
    }

    public void setItems(List<String> items)
    {
        this.items = items;
    }

    public boolean hasItem(ShopItem item)
    {
        return items.contains(item.getDataName());
    }

    public void removeItem(ShopItem item)
    {
        items.remove(item.getDataName());
    }

    public boolean isMasterBuilder()
    {
        return masterBuilder;
    }

    public boolean hasInspection()
    {
        return inspect;
    }

    public Map<String, Object> toSQLStorable()
    {
        Map<String, Object> objectMap = new HashMap<>();
        objectMap.put("uuid", uuid.toString());
        objectMap.put("ips", FUtil.listToString(ips));
        objectMap.put("notes", FUtil.listToString(notes));
        objectMap.put("tag", FUtil.miniMessage(tag));
        objectMap.put("discord_id", discordID);
        objectMap.put("master_builder", masterBuilder);
        objectMap.put("ride_mode", rideMode.name());
        objectMap.put("coins", coins);
        objectMap.put("items", FUtil.listToString(items));
        objectMap.put("total_votes", totalVotes);
        objectMap.put("display_discord", displayDiscord);
        objectMap.put("login_message", loginMessage);
        objectMap.put("inspect", inspect);
        return objectMap;
    }

    public boolean doesDisplayDiscord()
    {
        return displayDiscord;
    }

    public UUID getUuid()
    {
        return uuid;
    }

    public String getName()
    {
        return Bukkit.getOfflinePlayer(uuid).getName();
    }

    public Component getTag()
    {
        return tag;
    }

    public void setTag(Component tag)
    {
        this.tag = tag;
    }

    public String getDiscordID()
    {
        return discordID;
    }

    public void setDiscordID(String discordID)
    {
        this.discordID = discordID;
    }

    public Boolean getMasterBuilder()
    {
        return masterBuilder;
    }

    public void setMasterBuilder(Boolean masterBuilder)
    {
        this.masterBuilder = masterBuilder;
    }

    public RideMode getRideMode()
    {
        return rideMode;
    }

    public void setRideMode(RideMode rideMode)
    {
        this.rideMode = rideMode;
    }

    public int getCoins()
    {
        return coins;
    }

    public void setCoins(int coins)
    {
        this.coins = coins;
    }

    public int getTotalVotes()
    {
        return totalVotes;
    }

    public void setTotalVotes(int totalVotes)
    {
        this.totalVotes = totalVotes;
    }

    public boolean isDisplayDiscord()
    {
        return displayDiscord;
    }

    public void setDisplayDiscord(boolean displayDiscord)
    {
        this.displayDiscord = displayDiscord;
    }

    public String getLoginMessage()
    {
        return loginMessage;
    }

    public void setLoginMessage(String loginMessage)
    {
        this.loginMessage = loginMessage;
    }

    public Boolean getInspect()
    {
        return inspect;
    }

    public void setInspect(Boolean inspect)
    {
        this.inspect = inspect;
    }

    public enum RideMode
    {
        ASK,
        NORMAL,
        OFF
    }
}