package me.totalfreedom.totalfreedommod.util;

import me.totalfreedom.totalfreedommod.api.Interpolator;
import org.bukkit.Color;

import java.util.LinkedHashSet;

public class Interpolation
{
    public LinkedHashSet<Color> rainbow(int length)
    {
        LinkedHashSet<Color> base = new LinkedHashSet<>();
        LinkedHashSet<Color> redToOrange = hsvGradient(length, Color.RED, Color.ORANGE, this::linear);
        LinkedHashSet<Color> orangeToYellow = hsvGradient(length, Color.ORANGE, Color.YELLOW, this::linear);
        LinkedHashSet<Color> yellowToGreen = hsvGradient(length, Color.YELLOW, Color.GREEN, this::linear);
        LinkedHashSet<Color> greenToBlue = hsvGradient(length, Color.GREEN, Color.BLUE, this::linear);
        LinkedHashSet<Color> blueToPurple = hsvGradient(length, Color.BLUE, Color.PURPLE, this::linear);
        LinkedHashSet<Color> purpleToRed = hsvGradient(length, Color.PURPLE, Color.RED, this::linear);
        base.addAll(redToOrange);
        base.addAll(orangeToYellow);
        base.addAll(yellowToGreen);
        base.addAll(greenToBlue);
        base.addAll(blueToPurple);
        base.addAll(purpleToRed);
        return base;
    }

    private double[] linear(double from, double to, int max)
    {
        final double[] res = new double[max];
        for (int i = 0; i < max; i++)
        {
            res[i] = from + i * ((to - from) / (max - 1));
        }
        return res;
    }

    private LinkedHashSet<Color> hsvGradient(int length, Color from, Color to, Interpolator interpolator)
    {
        // returns a float-array where hsv[0] = hue, hsv[1] = saturation, hsv[2] = value/brightness
        final float[] hsvFrom = java.awt.Color.RGBtoHSB(from.getRed(), from.getGreen(), from.getBlue(), null);
        final float[] hsvTo = java.awt.Color.RGBtoHSB(to.getRed(), to.getGreen(), to.getBlue(), null);

        final double[] h = interpolator.interpolate(hsvFrom[0], hsvTo[0], length);
        final double[] s = interpolator.interpolate(hsvFrom[1], hsvTo[1], length);
        final double[] v = interpolator.interpolate(hsvFrom[2], hsvTo[2], length);

        final LinkedHashSet<Color> gradient = new LinkedHashSet<>();

        for (int i = 0; i < length; i++)
        {
            final int rgb = java.awt.Color.HSBtoRGB((float) h[i], (float) s[i], (float) v[i]);
            final Color color = Color.fromRGB(rgb);
            gradient.add(color);
        }
        return gradient;
    }
}
